@@ -1,6 +1,6 @@
 package Catalyst::View::ByCode::Renderer;
-BEGIN {
-  $Catalyst::View::ByCode::Renderer::VERSION = '0.15';
+{
+  $Catalyst::View::ByCode::Renderer::VERSION = '0.16';
 }
 use strict;
 use warnings;
@@ -29,6 +29,34 @@ our %EXPORT_TAGS = (
     default => [ @EXPORT ],
 );
 
+our @IS_KNOWN = (
+    # HTML5 tags not defined in HTML::Tagset
+    qw( article aside audio
+        canvas command
+        datalist details
+        figcaption figure footer
+        header hgroup
+        keygen
+        markup meter
+        nav
+        output 
+        progress
+        rt ruby
+        section source summary
+        time
+        video ),
+    grep { m{\A \w}xms }
+    keys(%HTML::Tagset::isKnown)
+);
+
+our %EMPTY_ELEMENT = (
+    (
+        map { ($_=>1) }
+        qw(source) ### FIXME: more needed!!!
+    ),
+    %HTML::Tagset::emptyElement
+);
+
 #
 # define variables -- get local() ized at certain positions
 #
@@ -132,7 +160,10 @@ sub overloaded_import {
         my %declare;
         
         foreach my $symbol (@{"$imported_package\::EXPORT_BLOCK"}) {
+            ### FIXME: aliasing makes trouble in case of overwriting !!!!!
             *{"$calling_package\::$symbol"} = *{"$imported_package\::$symbol"};
+            # *{"$calling_package\::$symbol"} = eval qq{ sub { goto $imported_package\::$symbol } };
+            # *{"$calling_package\::$symbol"} = eval qq{ sub { $imported_package\::$symbol(\@_) } };
             
             $declare{$symbol} = {
                 const => Catalyst::View::ByCode::Declare::tag_parser
@@ -260,7 +291,7 @@ sub _render {
                        ) .
                        
                        # closing tag or content?
-                       (exists($HTML::Tagset::emptyElement{$_->[0]})
+                       (exists($EMPTY_ELEMENT{$_->[0]})
                           ? ' />'
                           : '>' . 
                             _render(@{$_}[2 .. $#$_]) .
@@ -282,8 +313,27 @@ sub _render {
 sub template(&) {
     my $package = caller;
     
+    my $code = shift;
     no strict 'refs';
-    *{"$package\::RUN"} = $_[0];
+    *{"$package\::RUN"} = sub {
+        push @{$top[-1]}, [ '', {} ];
+        
+        push @top, $top[-1]->[-1];
+        
+        my $text = $code->();
+        if (ref($text) && UNIVERSAL::can($text, 'render')) {
+            push @{$top[-1]}, $text->render;
+        } else {
+            no warnings 'uninitialized'; # we might see undef values
+            $text =~ s{($NEED_ESCAPE)}{'&#' . ord($1) . ';'}oexmsg;
+            push @{$top[-1]}, $text;
+        }
+        
+        pop @top;
+        return;
+    };
+    
+    return;
 }
 
 #
@@ -584,6 +634,8 @@ HTML
     $_[0]->() if ($_[0]);
     
     push @{$top[-1]}, '</html>';
+    
+    return;
 }
 
 ######################################## Locale stuff
@@ -608,8 +660,7 @@ sub _construct_functions {
     my %declare;
     
     # tags with content are treated the same as tags without content
-    foreach my $tag_name (grep { m{\A \w}xms }
-                          keys(%HTML::Tagset::isKnown)) {
+    foreach my $tag_name (@IS_KNOWN) {
         my $sub_name = $change_tags{$tag_name}
             || $tag_name;
         
